/*
 * @Author: LiZhiWei
 * @Date: 2025-05-14 15:46:50
 * @LastEditors: LiZhiWei
 * @LastEditTime: 2025-05-14 15:50:02
 * @Description:
 */
// 创建图表元素
export function createChartElement (element) {
  // 1. 创建基础容器
  const el = document.createElement('div')
  el.style.position = 'absolute'
  el.style.top = element.top + 'px'
  el.style.left = element.left + 'px'
  el.style.width = element.width + 'px'
  el.style.height = element.height + 'px'
  el.style.zIndex = element.order
  // 2. 创建SVG画布
  const svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg')
  svg.setAttribute('width', element.width)
  svg.setAttribute('height', element.height)
  svg.setAttribute('viewBox', `0 0 ${element.width} ${element.height}`)

  // 3. 设置图表内边距
  const padding = {
    top: 60, // 为图例留出空间
    right: 40, // 右侧边距
    bottom: 60, // X轴标签空间
    left: 60 // Y轴标签空间
  }

  // 4. 计算实际绘图区域
  const chartWidth = element.width - padding.left - padding.right
  const chartHeight = element.height - padding.top - padding.bottom
  // 处理不同图表类型
  switch (element.chartType) {
    case 'barChart':
      // 绘制柱状图
      drawBarChart(svg, element, {
        padding,
        chartWidth,
        chartHeight,
        barDir: element.barDir || 'col',
        grouping: element.grouping || 'clustered'
      })
      break
    case 'pieChart':
    case 'doughnutChart':
      drawDonutChart(svg, element, chartWidth, chartHeight)
      break
    default:
      console.warn('Unsupported chart type:', element.chartType)
  }

  el.appendChild(svg)
  return el
}

export function drawDonutChart (svg, element, chartWidth, chartHeight) {
  // 计算饼图中心点和半径
  const cx = element.width / 2
  const cy = element.height / 2
  const radius = Math.min(chartWidth, chartHeight) / 2

  // 计算所有数据总和
  const total = element.data[0].values.reduce((sum, item) => sum + item.y, 0)

  // 创建颜色数组
  const colors = element.colors || ['#5B9BD5', '#ED7D31', '#A5A5A5', '#FFC000']

  let startAngle = 0
  element.data[0].values.forEach((item, index) => {
    // 计算扇形角度
    const angle = (item.y / total) * Math.PI * 2

    // 计算扇形路径
    const endAngle = startAngle + angle
    const x1 = cx + radius * Math.cos(startAngle)
    const y1 = cy + radius * Math.sin(startAngle)
    const x2 = cx + radius * Math.cos(endAngle)
    const y2 = cy + radius * Math.sin(endAngle)

    // 创建扇形路径
    const path = document.createElementNS('http://www.w3.org/2000/svg', 'path')
    const largeArcFlag = angle > Math.PI ? 1 : 0

    // 根据是否为环形图决定内圆半径
    const innerRadius = element.chartType === 'doughnutChart' ? radius * 0.6 : 0
    if (element.chartType === 'doughnutChart') {
      // 环形图路径
      const ix1 = cx + innerRadius * Math.cos(startAngle)
      const iy1 = cy + innerRadius * Math.sin(startAngle)
      const ix2 = cx + innerRadius * Math.cos(endAngle)
      const iy2 = cy + innerRadius * Math.sin(endAngle)

      path.setAttribute(
        'd',
        `
          M ${x1} ${y1}
          A ${radius} ${radius} 0 ${largeArcFlag} 1 ${x2} ${y2}
          L ${ix2} ${iy2}
          A ${innerRadius} ${innerRadius} 0 ${largeArcFlag} 0 ${ix1} ${iy1}
          Z
        `
      )
    } else {
      // 饼图路径
      path.setAttribute(
        'd',
        `
          M ${cx} ${cy}
          L ${x1} ${y1}
          A ${radius} ${radius} 0 ${largeArcFlag} 1 ${x2} ${y2}
          Z
        `
      )
    }

    // 设置填充色
    path.setAttribute('fill', colors[index % colors.length])
    path.setAttribute('stroke', '#fff')
    path.setAttribute('stroke-width', '1')
    svg.appendChild(path)

    // 添加数值标签
    const labelAngle = startAngle + angle / 2
    const labelRadius = radius * 1.1
    const labelX = cx + labelRadius * Math.cos(labelAngle)
    const labelY = cy + labelRadius * Math.sin(labelAngle)

    const label = document.createElementNS('http://www.w3.org/2000/svg', 'text')
    label.setAttribute('x', labelX)
    label.setAttribute('y', labelY)
    label.setAttribute('text-anchor', 'middle')
    label.setAttribute('alignment-baseline', 'middle')
    label.setAttribute('font-size', '12px')
    label.textContent = `${((item.y / total) * 100).toFixed(1)}%`
    svg.appendChild(label)

    startAngle = endAngle
  })
}
// 绘制柱状图
export function drawBarChart (svg, element, options) {
  const { padding, chartWidth, chartHeight, barDir, grouping } = options
  const series = element.data
  const categories = series[0].xlabels
  const categoryCount = Object.keys(categories).length

  // 1. 计算最大值
  let maxValue = 0
  series.forEach((serie) => {
    const seriesMax = Math.max(...serie.values.map((v) => v.y))
    maxValue = Math.max(maxValue, seriesMax)
  })
  maxValue = maxValue * 1.2 // 增加20%空间

  // 2. 计算柱子布局
  const groupWidth = chartWidth / categoryCount
  const barWidth =
      grouping === 'clustered'
        ? (groupWidth * 0.6) / series.length // 分组模式
        : groupWidth * 0.6 // 堆叠模式
  const barSpacing = (groupWidth * 0.4) / (grouping === 'clustered' ? series.length + 1 : 2)

  // 3. 绘制坐标轴
  // X轴
  const xAxisPath = document.createElementNS('http://www.w3.org/2000/svg', 'path')
  xAxisPath.setAttribute(
    'd',
    `M${padding.left},${element.height - padding.bottom} L${element.width - padding.right},${
      element.height - padding.bottom
    }`
  )
  xAxisPath.setAttribute('stroke', '#000')
  xAxisPath.setAttribute('stroke-width', '1')
  svg.appendChild(xAxisPath)

  // Y轴
  const yAxisPath = document.createElementNS('http://www.w3.org/2000/svg', 'path')
  yAxisPath.setAttribute(
    'd',
    `M${padding.left},${padding.top} L${padding.left},${element.height - padding.bottom}`
  )
  yAxisPath.setAttribute('stroke', '#000')
  yAxisPath.setAttribute('stroke-width', '1')
  svg.appendChild(yAxisPath)

  // 4. 绘制Y轴刻度和网格线
  const yTickCount = 5
  for (let i = 0; i <= yTickCount; i++) {
    const y = padding.top + (chartHeight * i) / yTickCount
    const value = maxValue - (maxValue * i) / yTickCount

    // 水平网格线
    const gridLine = document.createElementNS('http://www.w3.org/2000/svg', 'line')
    gridLine.setAttribute('x1', padding.left)
    gridLine.setAttribute('y1', y)
    gridLine.setAttribute('x2', padding.left + chartWidth)
    gridLine.setAttribute('y2', y)
    gridLine.setAttribute('stroke', '#eee')
    gridLine.setAttribute('stroke-width', '1')
    svg.appendChild(gridLine)

    // 刻度线
    const tick = document.createElementNS('http://www.w3.org/2000/svg', 'line')
    tick.setAttribute('x1', padding.left - 6)
    tick.setAttribute('y1', y)
    tick.setAttribute('x2', padding.left)
    tick.setAttribute('y2', y)
    tick.setAttribute('stroke', '#000')
    tick.setAttribute('stroke-width', '1')
    svg.appendChild(tick)

    // 刻度值
    const label = document.createElementNS('http://www.w3.org/2000/svg', 'text')
    label.setAttribute('x', padding.left - 10)
    label.setAttribute('y', y + 4)
    label.setAttribute('text-anchor', 'end')
    label.setAttribute('font-size', '12px')
    label.textContent = value.toFixed(1)
    svg.appendChild(label)
  }

  // 5. 绘制数据条
  series.forEach((serie, serieIndex) => {
    serie.values.forEach((value, index) => {
      const barHeight = (value.y / maxValue) * chartHeight
      const x =
          padding.left +
          groupWidth * index +
          (grouping === 'clustered'
            ? barSpacing * (serieIndex + 1) + barWidth * serieIndex
            : barSpacing)
      const y = element.height - padding.bottom - barHeight

      // 绘制柱子
      const bar = document.createElementNS('http://www.w3.org/2000/svg', 'rect')
      bar.setAttribute('x', x)
      bar.setAttribute('y', y)
      bar.setAttribute('width', barWidth)
      bar.setAttribute('height', barHeight)
      bar.setAttribute(
        'fill',
        element.colors[serieIndex] || `hsl(${serieIndex * 60}, 70%, 50%)`
      )
      svg.appendChild(bar)

      // 数值标签
      if (element.marker) {
        const label = document.createElementNS('http://www.w3.org/2000/svg', 'text')
        label.setAttribute('x', x + barWidth / 2)
        label.setAttribute('y', y - 5)
        label.setAttribute('text-anchor', 'middle')
        label.setAttribute('font-size', '12px')
        label.textContent = value.y.toFixed(1)
        svg.appendChild(label)
      }
    })
  })

  // 6. 绘制X轴类别标签
  Object.values(categories).forEach((label, index) => {
    const x = padding.left + groupWidth * (index + 0.5)
    const text = document.createElementNS('http://www.w3.org/2000/svg', 'text')
    text.setAttribute('x', x)
    text.setAttribute('y', element.height - padding.bottom + 20)
    text.setAttribute('text-anchor', 'middle')
    text.setAttribute('font-size', '12px')
    text.textContent = label
    svg.appendChild(text)
  })

  // 7. 绘制图例
  series.forEach((serie, index) => {
    const legendX = padding.left + index * 120
    const legendY = 20

    const rect = document.createElementNS('http://www.w3.org/2000/svg', 'rect')
    rect.setAttribute('x', legendX)
    rect.setAttribute('y', legendY)
    rect.setAttribute('width', 15)
    rect.setAttribute('height', 15)
    rect.setAttribute('fill', element.colors[index] || `hsl(${index * 60}, 70%, 50%)`)
    svg.appendChild(rect)

    const text = document.createElementNS('http://www.w3.org/2000/svg', 'text')
    text.setAttribute('x', legendX + 25)
    text.setAttribute('y', legendY + 12)
    text.setAttribute('font-size', '12px')
    text.textContent = serie.key
    svg.appendChild(text)
  })
}
